# OpenDDS and Android

**WARNING: Android support is experimental and not fully supported!**

## Building OpenDDS for Android

Building OpenDDS for Android requires the [Android Native Development Kit
(NDK)](https://developer.android.com/ndk/). As Android targets multiple
architectures and has many versions, an architecture and minimum API version to
use will have to be decided. As of writing [this page lists Android version
numbers and their corresponding API
versions.](https://source.android.com/setup/start/build-numbers)

NDK r18 or higher is required to build OpenDDS for Android.

*NOTE: These examples are for Unix shell and assume $NDK is the location of the
unarchived NDK, $TOOLCHAIN is set to the location of the toolchain, and
$DDS_ROOT is the location of OpenDDS.*

A standalone toolchain is required and can be generated by using
`build/tools/make_standalone_toolchain.py` Python script in the NDK.
For example, to create a toolchain for 32-bit ARM Android 7.0 "Nougat" and
later:

```
$NDK/build/tools/make_standalone_toolchain.py --arch arm --api 24 --install-dir $TOOLCHAIN
```

Once a toolchain is obtained, OpenDDS can be configured to cross compile for
Android by passing `--target=android` and `--macros=ANDROID_ABI=<ARCH>` to the
OpenDDS configure script. The `--api` argument for
`make_standalone_toolchain.py` and `--macros=ANDROID_ABI=<ARCH>` argument for
the configure script must match according to this table:

| `--arch` | `ANDROID_ABI`           | Description                                  |
| -------- | ----------------------- | -------------------------------------------- |
| `arm`    | `armeabi-v7a`           | 32-bit ARM                                   |
| `arm`    | `armeabi-v7a-with-neon` | `armeabi-v7a` with NEON extensions enabled   |
| `arm64`  | `arm64-v8a`             | 64-bit ARM, Sometimes referred to as aarch64 |
| `x86`    | `x86`                   | 32-bit x86                                   |
| `x86_64` | `x86_64`                | 64-bit x86                                   |

To build OpenDDS with the toolchain generated in the previous example:

```
cd $DDS_ROOT
./configure --no-tests --target=android --macros=ANDROID_ABI=armeabi-v7a
PATH=$PATH:$TOOLCHAIN/bin make
```

### OpenDDS's Optional Dependencies

<!--
TODO
#### Java
-->

#### OpenSSL

OpenSSL is required for OpenDDS Security. Fortunately it comes with a
configuration for android, please read `NOTES.ANDROID` for more information.

NOTE: As of OpenSSL 1.1.1, the script for Android requires the NDK itself, not
a standalone toolchain generated from the NDK. Specifically it requires
`$ANDROID_NDK` to be set to the location of the NDK and the toolchain (not the
target specific toolchain) be added to `$PATH`. For example this would be
`$ANDROID_NDK/toolchains/llvm/prebuilt/linux-x86_64/bin` for x86\_64 Linux host
for NDK r18.

#### Xerces

Xerces is also required for OpenDDS Security. Unfortunately it does not support
Android specifically but should build after some coaxing. In the authors
case it required modifying Xerces' configure script to skip checking for
`wcsrtombs()` and assume it was there.

<!--
TODO
## Using OpenDDS with C++ on Android

## Using OpenDDS with Java on Android

## Running Tests
-->
